-- game bot AI
AI = {}

-- function, that loads bots
function AI:load()
	BOT = {}							-- BOT list
	i = 1	-- bot count
	-- import from tilemap all bots
	if map.objects.obj['bot'] then
		math.randomseed(os.time())
		for k,v in ipairs(map.objects.obj['bot']) do
			BOT[i] = {
			
			-- random bot variables
			type = 'lbot',					-- change to loaded bot
			lvl = v.properties.lvl,
			isSpawned = false,
			
			-- physics
			startPos = v.pos:clone(),		-- start position, never changes
			pos = v.pos:clone(),			-- current position vector
			shape = vector(64, 64),			-- shape vector
			disp = v.pos:clone(), 			-- position after calculation
			gravity = vector(0, 300),		-- gravity
			vel = vector(0,0),				-- velocity
			force = vector(0,0),			-- force that accumulates per frame
			damp = 0.98,					-- friction		
			
			-- movement
			canJump = false,				-- can BOT jump?
			rightLen = v.properties.rightLen,-- how much bot can move to right in pixels
			leftLen = v.properties.leftLen,	-- how much bot can move to left in pixels
			dir = 1,--math.floor(math.random() + 0.49),	-- movement direction. -1 left, 1 right
			speed = v.properties.speed,		-- movement speed
			
			-- graphics
			spriteSheet = {},
			walkAni = {}
		}
		
		-- adding correct animation by bot lvl
		BOT[i].spriteSheet = love.graphics.newImage(img..'bot_'..BOT[i].lvl..'.png')
		BOT[i].walkAni = newAnimation(BOT[i].spriteSheet, BOT[i].shape.x, BOT[i].shape.y, 0.1, 5)	
		BOT[i].pos.y = BOT[i].pos.y - BOT[i].shape.x	
		hash:insert(BOT[i])
		i = i + 1
		end
	end
--	map.objects.obj['bot'] = nil
end

-- function, that draws bots
function AI:draw(camx, camy)
	-- for all bots
	for i, v in pairs (BOT) do
		if BOT[i].dir == 1 then
			BOT[i].walkAni:draw(
				BOT[i].pos.x + mapper.hscreen_w,
				BOT[i].pos.y + mapper.hscreen_h,
				0,
				BOT[i].dir,
				1
			)
		else
			BOT[i].walkAni:draw(
				BOT[i].pos.x + mapper.hscreen_w + BOT[i].shape.x,
				BOT[i].pos.y + mapper.hscreen_h,
				0,
				BOT[i].dir,
				1
			)
		end
	end
end

-- local function that tells what to collide with
local function can_collide(self, with)
	if		with == 'wall' or
			with == 'platform' or
			with == 'hero' or
			with == 'iceblock' or
			with == 'breakable' then
		return true
	end
	return false
end

-- local function for bot collision
local function collide(self, hash)
	local set = hash:getNeighbours(self)
	local collided = false
	for _,other in pairs(set) do
		if can_collide(self, other.type) then
			if standon(self, other.pos, other.shape) then
				-- reseting velocity, so we can jump easily
				self.vel.y = 0
				collided = true
				self.spawned = true;
			end
			if nearwall(self, other.pos, other.shape) and other.type == 'hero' then
				other:gethit(1)
			end
		end
	end
	-- interactions here
end

-- function, that updates bots
function AI:update(dt, hash)
	for i, v in pairs (BOT) do
		-- checking if bot reached moving limits
		if ((BOT[i].dir == 1) and (BOT[i].pos.x >= BOT[i].startPos.x + BOT[i].rightLen)) or
			((BOT[i].dir == -1) and (BOT[i].pos.x <= BOT[i].startPos.x - BOT[i].leftLen)) then
				BOT[i].dir = BOT[i].dir * -1
		end

		-- move bot
		if BOT[i].spawned then
			applyForce(BOT[i], BOT[i].dir * BOT[i].speed, 0)
		end

		-- collision stuff
		phys1(BOT[i], dt)
		collide(BOT[i], hash)
		phys2(BOT[i], hash)
		
		-- update animations
		BOT[i].walkAni:update(dt)
	end
end

-- function that removes "other" bot object from BOT array and hash
function AI:remove(other)
	for k, v in pairs (BOT) do
		if v == other then
			table.remove(BOT, k)
		end
	end
	hash:remove(other)
end
