-- idea here is simple:
-- objects should have top left corner position vector (.pos)
-- and size of object as vector from top left to bottom right corner (.shape)
-- OR you have to explicically give ul and lr corners
--
-- MUSTS:
--	require('SHWrap')
--	some_val = SHWrap([cell_size])
--		=== cell_size - how big is internal geographical grid..
--		=== use sane number such 'double the size of average object'
--
--
-- you add ONCE object you need to track of with SHWrap:insert(obj [, ul, lr])
--
-- call SHWrap:getNeighbours(obj [, ul, lr])
-- to get a set (table) of objects in same cells as given obj
--		=== once you get your neighbour list, you can do your collision checks,
--		=== or any other kind of magic you would want to do
--
-- call SHWrap:update(obj, new_ul [, old_ul, new_lr, old_lr])
-- for every object that is moving
--
-- SHWrap:remove(obj, [, ul, lr]) if you want to untrack it from world

local _PACK = (...):match("(.*)/([^/.]-)$")
local vector = require(_PACK .. '/vector')
local Spatialhash = require(_PACK .. '/spatialhash')
if not (common and common.class and common.instance) then
	class_commons = true
	require(_PACK .. '/class')
end


-- for allmost all subsequent functions: use explicitic shape size if needed
local function insert(self, obj, ul, lr)
	self._hash:insert(obj, ul or obj.pos, lr or obj.pos + obj.shape)
end

local function remove(self, obj, ul, lr)
	self._hash:remove(obj, ul or obj.pos, lr or obj.pos + obj.shape)
end

local function update(self, obj, new_ul,    old_ul, new_lr, old_lr)
	self._hash:update(obj,
			old_ul or obj.pos,
			nil ~= old_ul and (old_lr or old_ul + obj.shape) or obj.pos + obj.shape,
			new_ul,
			new_lr or new_ul + obj.shape)
end

local function getNeighbours(self, obj, ul, lr)
	return self._hash:getNeighbors(obj, ul or obj.pos, lr or obj.pos + obj.shape)
end

local function draw(self, how, show_empty, print_key)
	self._hash:draw(how, show_empty, print_key)
end

function SHWrap(cell_size)
--	local SHHash = {}
	local _csize = cell_size or 120

	return setmetatable({
		_csize = _csize,
		_hash = common.instance(Spatialhash, _csize),
		insert = insert,
		remove = remove,
		update = update,
		getNeighbours = getNeighbours,
		draw = draw,
	}, {})
end

return { SHWrap = SHWrap }
