mapper = {}

local name_patt = "name=\"(.-)\""
local w_patt = "width=\"(%d+)\""
local h_patt = "height=\"(%d+)\""
local prop_patt = '<property name=\"(%w-)\" value=\"(.-)\"/>'
local box_patt = '<object type=\"(%w-)\" x=\"(%d+)\" y=\"(%d+)\" width=\"(%d+)\" height=\"(%d+)\"/>'
mapper.draworder = {
	['background'] = 3,
	['foreground'] = 10,
	['objects'] = 5,
	['decorations'] = 4,
}
mapper.resources = {}
-- render in intel gma compactability mode by default
mapper.GGMA = true

--collision parser
local function _get_collision_map(data)
	for name, xarg in data:gmatch('<objectgroup name=\"(collisions)\"(.-)</objectgroup>') do
		map[name] = {}
		for typ, x, y, w, h in xarg:gmatch(box_patt) do
			x, y = tonumber(x), tonumber(y)
			w, h = tonumber(w), tonumber(h)
			if nil == map[name][typ] then
				map[name][typ] = {}
			end
			table.insert(map[name][typ], {
				pos = vector(x, y),
				shape = vector(w, h),
				type = typ,
			}
			)
		end   
		--parse teleport
		local patt = '<object type=\"(%w+)\" x=\"(%d+)\" y=\"(%d+)\" width=\"(%d+)\" height=\"(%d+)\">%s+<properties>%s+(.-)%s+</properties>%s+</object>'
		for typ, xx, yy, ww, hh, prop in xarg:gmatch(patt) do
			xx, yy, ww, hh = tonumber(xx), tonumber(yy), tonumber(ww), tonumber(hh)
			local properties = {}
			for nam, val in prop:gmatch(prop_patt) do
				properties[nam] = val
			end
			if nil == map[name][typ] then
				map[name][typ] = {}
			end
			table.insert(map[name][typ], {
				tid = id,
				properties = properties,
				pos = vector (xx, yy),
				shape = vector (ww, hh),
				type = typ,
				})
		end
	end
end

local function reg_tile(tileid, lmap, tilearr)
	local gid = #tilearr
	while tileid < tilearr[gid] do
		gid = gid - 1
	end
	lmap.tile[tileid] = {
		tileset = tilearr[gid],
	}
	gid = tileid - tilearr[gid]
	local tsetid = lmap.tile[tileid].tileset
	local row = math.floor(gid / lmap.tileset[tsetid].coln)
	local col = gid  - row * lmap.tileset[tsetid].coln
--	lmap.tile[tileid].x = col * lmap.tileset[lmap.tile[tileid].tileset].tilew
--	lmap.tile[tileid].y = row * lmap.tileset[lmap.tile[tileid].tileset].tileh
	lmap.tile[tileid].q = love.graphics.newQuad(
		col * lmap.tileset[tsetid].tilew,  row * lmap.tileset[tsetid].tileh,
		lmap.tileset[tsetid].tilew,         lmap.tileset[tsetid].tileh,
--		lmap.tileset[tsetid].width,         lmap.tileset[tsetid].height
		lmap.tileset[tsetid].src:getWidth(),	lmap.tileset[tsetid].src:getHeight()
		)
end


local function _get_tilearr(lmap)
	local key = {}
	for i,_ in pairs(lmap.tileset) do
		table.insert(key, i)
	end
	table.sort(key)
	return key
end

local function _levelsplit (str, lmap)

	local key = _get_tilearr(map)

	local tmap = {}
	local last_tile
	if str:len() > 0 then
		nLast = 0
		local nLast, tid = 0, 1
		patt = '(%d+)'
		for rowField=1, lmap.height do
			local rowmap = {}
			for colField=1, lmap.width do
				_, nLast, tid = str:find(patt, nLast+1)
				tid = tonumber(tid)
				if 0 ~= tid then
					rowmap[colField] = tid
					last_tile = tid
					if nil == lmap.tile[tid] then
						reg_tile(tid, lmap, key)
					end
				end
			end
			tmap[rowField] = rowmap
		end
	end
	return tmap--, lmap.tile[last_tile].tileset
end

local function _get_tilesets(data)
	local t_patt = '<tile id=\"(%d+)\">(.-)</tile>'
	for tset, xarg in data:gmatch('<tileset (.-)>(.-)</tileset>') do
		local tileset = {
			firstid = tonumber(tset:match("firstgid=\"(%d+)\"")),
			name = tset:match(name_patt),
			tilew = tonumber(tset:match("tilewidth=\"(%d+)\"")),
			tileh = tonumber(tset:match("tileheight=\"(%d+)\"")),
			width = tonumber(xarg:match(w_patt)),
			height = tonumber(xarg:match(h_patt)),
			src_name = xarg:match("source=\"(.-)\""),
			trans = xarg:match("trans=\"(.-)\""),
		}
		tileset.trans = {
			tonumber('0x'..tileset.trans:sub(1,2)),
			tonumber('0x'..tileset.trans:sub(3,4)),
			tonumber('0x'..tileset.trans:sub(5,6)),
		}
		tileset.rown = math.floor(tileset.height / tileset.tileh)
		tileset.coln = math.floor(tileset.width / tileset.tilew)
		tileset.tshape = vector(tileset.tilew, tileset.tileh)
		-- other extra possible arguments
		for tile, prop in xarg:gmatch(t_patt) do
			-- take generic tile properties & register tile to tile table
			tile = tonumber(tile)
			map.tile[tileset.firstid + tile] = { tileset = tileset.firstid }
			local row = math.floor(tile / tileset.coln)
			local col = tile - row * tileset.coln
			map.tile[tileset.firstid + tile].q = love.graphics.newQuad(
				col * tileset.tilew, row * tileset.tileh,
				tileset.tilew, tileset.tileh,
--				tileset.width, tileset.height
				tileset.src:getWidth(), tileset.src:getHeight()

				)
			map.tile[tileset.firstid + tile].properties = {}
			for nam, val in prop:gmatch(prop_patt) do
				map.tile[tileset.firstid + tile].properties[nam] = val
			end
		end
		-- save generated tileset
		map.tileset[tileset.firstid] = tileset
	end
end

local function _get_map_resources()
	local pre = mapper.asset_path
	for k,v in pairs(map.tileset) do
		-- if we have already loaded that tileset, lets use it
		if mapper.resources[ v.src_name ] then
			v.src = mapper.resources[ v.src_name ]
		else
			-- do give me transparency
			map.tileset[k].src = love.image.newImageData(pre .. v.src_name)
			-- makes images transparent
			map.tileset[k].src:mapPixel(
				function(x, y, r, g, b, a)
					if (r == v.trans[1] and g == v.trans[2] and b == v.trans[3]) then
						a = 0
					end
					return r, g, b, a
				end
			)
			-- power of padding.. who in the name of... though of using not PO2
			-- images... anyways...
			local wp = math.pow(2, math.ceil(math.log(v.width)/math.log(2)))
			local hp = math.pow(2, math.ceil(math.log(v.height)/math.log(2)))
			if wp ~= v.width or hp ~= v.height then
				local pad = love.image.newImageData(wp, hp)
				pad:paste(v.src, 0, 0, 0, 0, v.width, v.height)
				v.src = pad
			end
			v.src = love.graphics.newImage(v.src)
			-- lets remember file for later use
			mapper.resources[ v.src_name ] = v.src
		end
	end
	-- not even worth creating sprite batch if we are on intel GMA
	if not mapper.GGMA then
		for k,v in pairs(map.layer) do
			local tset = map.tileset[v.tileset]
			map.layer[k].batch = love.graphics.newSpriteBatch(
				tset.src, tset.rown * tset.coln
			)
			drawbuff:add(love.graphics.draw, mapper.draworder[k], v.batch)
		end
	end
end

local function _get_layers(data)
	local lay_patt = '<layer(.-)>(.-)</layer>'
	local data_patt = '<data.->(.-)</data>'
	for tset, xarg in data:gmatch(lay_patt) do
		local name = tset:match(name_patt)
		map.layer[name] = {
			width = tset:match(w_patt),
			height = tset:match(h_patt),
		}
		map.layer[name].grid, map.layer[name].tileset  = _levelsplit(xarg:match(data_patt),  map)
	end
end

local function _get_map_objects(data)
	for group, xarg in data:gmatch('<objectgroup(.-)>(.-)</objectgroup>') do
		local name = group:match(name_patt)
		if name ~= 'collisions' then
			map[name] = {
					width = group:match(w_patt),
					height = group:match(h_patt),
					obj = {},
				}
			local tilearr = _get_tilearr(map)
			-- read box objects w/o properties
			for typ, x, y, w, h in xarg:gmatch(box_patt) do
				x, y = tonumber(x), tonumber(y)
				w, h = tonumber(w), tonumber(h)
				if nil == map[name][typ] then
					map[name][typ] = {}
				end
				table.insert(map[name][typ], {
					pos = vector(x, y),
					shape = vector(w, h),
					type = typ,
				}
				)
			end
			-- read tile objects w/o properties
			local patt ='<object gid=\"(%d+)\" x=\"(%d+)\" y=\"(%d+)\"/>'
			for id, xx, yy in xarg:gmatch(patt) do
				id, xx, yy = tonumber(id), tonumber(xx), tonumber(yy)
				if nil == map.tile[id] then
					reg_tile(id, map, tilearr)
				end
				if nil == map[name].obj[xx] then
					table.insert(map[name].obj, xx, {})
				end
				--- fishy part here... may require change
				table.insert(map[name].obj[xx], yy, {tid = id})
			end
			-- reads tile w/ properties
			-- bot parser also (not only)
			local patt = '<object type=\"(%w-)\" gid=\"(%d+)\" x=\"(%d+)\" y=\"(%d+)\">\n(.-)</object>'
			for ttype, id, xx, yy, prop in xarg:gmatch(patt) do
				id, xx, yy = tonumber(id), tonumber(xx), tonumber(yy)
				local properties = {}
				for nam, val in prop:gmatch(prop_patt) do
					properties[nam] = val
				end
				if nil == map.tile[id] then
					reg_tile(id, map, tilearr)
				end
				if nil == map[name].obj[ttype] then
					map[name].obj[ttype] = {}
				end
				local tset = map.tile[id].tileset
				table.insert(map[name].obj[ttype], {
					tid = id,
					properties = properties,
					pos = vector (xx, yy - map.tileset[tset].tshape.y),
					shape = map.tileset[tset].tshape,
					type = ttype,
					})
			end
			-- yet another one other parser... I think I should finaly do a
			-- generic one... but I am lazy.
			local patt = '<object type=\"([%w ]-)\" gid=\"(%d+)\" x=\"(%d+)\" y=\"(%d+)\"/>'
			for ttype, id, xx, yy in xarg:gmatch(patt) do
				id, xx, yy = tonumber(id), tonumber(xx), tonumber(yy)
				if nil== map.tile[id] then
					reg_tile(id, map, tilearr)
				end
				if nil == map[name].obj[ttype] then
					map[name].obj[ttype] = {}
				end
				local tset = map.tile[id].tileset
				table.insert(map[name].obj[ttype], {
					tid = id,
					pos = vector(xx, yy - map.tileset[tset].tshape.y),
					shape = map.tileset[tset].tshape,
					type = ttype,
				})
			end
		end
	end
end


function mapper.load(filename)
	local data = love.filesystem.read(filename)
--	mapper.close()
	data:gsub('<map(.-)width=\"(%d+)\" height=\"(%d+)\" tilewidth=\"(%d+)\" tileheight=\"(%d+)\">',
			function (_, w, h, tw, th)
				map = {	width = tonumber(w),
						height = tonumber(h),
						tilew = tonumber(tw),
						tileh = tonumber(th),
						tileset = {},
						layer = {},
						tile = {},
				}
			end
			)
	-- read tilesets
	_get_tilesets(data)
	-- load required data
	_get_map_resources()
	-- read layers
	_get_layers(data)
	-- read object groups
--	_get_map_objects(data)
	local p = filename:split('.')[1]
	if not love.filesystem.isFile(p .. '.osd') then
		_get_map_objects(data)
	end
	_get_collision_map(data)
--	l = io.open('log', 'w')
--	l:write(stprint(map))
--	l:close()
	-- read collision data

	-- other stuff
	mapper.hscreen_w = love.graphics.getWidth() / 2
	mapper.hscreen_h = love.graphics.getHeight() / 2
	mapper.pmap_height = map.height * map.tileh
	mapper.pmap_width = map.width * map.tilew
	mapper.fbmax = 2048 / 2
	mapper.scale = 1
	mapper.old_fn = filename
	mapper.ox, mapper.oy = 0, 0
	mapper._gta = _get_tilearr
	mapper._reg_tile = reg_tile
end

function mapper.close()
	if nil ~= map then
		persistence.store(mapper.old_fn:split('.')[1] .. '.smd', map.objects)
	end
end

function mapper.drawNear(camx, camy)
	drawbuff:exec()
end

function mapper.removeobj(other)
	-- assuming other object is in map.objects.obj
	-- don't think too much, I am doing frigging;y many assumptions and magic
	for k,val in pairs(map.objects.obj[other.type]) do
		if other == val then
			table.remove(map.objects.obj[other.type], k)
		end
	end
end
gc = collectgarbage


function mapper.ready_layers()
	love.graphics.setColor(0xff, 0xff, 0xff, 0xff)
	local map_height = mapper.pmap_height
	local map_width = mapper.pmap_width
	local fbmax = mapper.fbmax
	local no_of_rows = math.floor(map_height / fbmax)
	local no_of_cols = math.floor(map_width / fbmax)
	local tid, tset
	static = {}
	for lname, ltable in pairs(map.layer) do
		static[lname] = {}

	for my=0,no_of_rows do
		static[lname][my * fbmax] = {}
		for mx=0,no_of_cols do
			local w, h
			-- get min power of 2 for frame buffer
			local left =map_height - my * fbmax
			if  left >= fbmax then
				h = fbmax
			else
				h = math.max(math.pow(2, math.ceil(math.log(left) / math.log(2))), map.tileh)
			end
			left = map_width - mx * fbmax
			if left >= fbmax then
				w = fbmax
			else
				w = math.max(math.pow(2, math.ceil(math.log(left) / math.log(2))), map.tilew)
			end
			local fb = love.graphics.newFramebuffer(w, h)
			local stpy = (my * fbmax) / map.tileh
			local stpx = (mx * fbmax) / map.tilew
			local enpy = math.ceil((my * fbmax + h) / map.tileh)
			local enpx = math.ceil((mx * fbmax + w) / map.tilew)
			love.graphics.setRenderTarget(fb)
			for y=stpy,enpy do
				if map.layer[lname].grid[y] then
					for x=stpx,enpx do
						if map.layer[lname].grid[y][x] then
							tid = map.layer[lname].grid[y][x]
							tset = map.tile[tid].tileset
							love.graphics.drawq(
								map.tileset[tset].src,
								map.tile[tid].q,
								(x-1 - stpx) * map.tilew,
								(y-1 - stpy) * map.tileh
							)
						end
					end
				end
			end
			static[lname][my * fbmax][mx * fbmax] = {
				fb = fb,
				w = w,
				h = h,
			}

		end
	end

	end
	map.layer = nil
	love.graphics.setRenderTarget()
end

function mapper.draw_layer(camx, camy, i)
	love.graphics.setColor(0xff,0xff,0xff,0xff)
	local stx = math.floor((camx - mapper.hscreen_w) / mapper.fbmax) * mapper.fbmax
	local enx = math.floor((camx + mapper.hscreen_w) / mapper.fbmax) * mapper.fbmax
	local sty = math.floor((camy - mapper.hscreen_h) / mapper.fbmax) * mapper.fbmax
	local eny = math.floor((camy + mapper.hscreen_h) / mapper.fbmax) * mapper.fbmax
	-- lets only draw the part we can see.. :)
	if static[i] then
		for y=sty,eny,mapper.fbmax do
			for x=stx,enx,mapper.fbmax do
				love.graphics.draw(
					static[i][y][x].fb,
					x + mapper.hscreen_w,
					y + mapper.hscreen_h
				)
			end
		end
	end
end





-- just as name says to string (serialization)
-- no userdata; no functions; supports vector
function mapper.save_objects(table_to_save)
	local objtmp = {}
	local nr
	table.insert(objtmp, '{\n')
	for k,v in pairs(table_to_save) do
		nr = 'number' == type(k) and '[' .. k .. ']=' or '["' .. k .. '"]='
		if 'number' == type(v) then
			table.insert(objtmp, nr)
			table.insert(objtmp, v .. ';\n')
		elseif 'string' == type(v) then
			table.insert(objtmp, nr)
			table.insert(objtmp, '"' .. v .. '";\n')
		elseif 'boolean' == type(v) then
			table.insert(objtmp, nr)
			table.insert(objtmp, tostring(v) .. ';\n')
		elseif 'table' == type(v) then
			if v.strnize then
				table.insert(objtmp, nr)
				table.insert(objtmp, v:strnize() .. ';\n')
			else
				local s, c = mapper.save_objects(v)
				if c then
					table.insert(objtmp, nr)
					table.insert(objtmp, s)
				end
			end
		end
	end
	table.insert(objtmp, '};\n')
	return table.concat(objtmp), #objtmp
end

function mapper.reg_missing_obj_tiles(table_to_rereg)
	local ta = mapper._gta(map)
	for typ,v in pairs(table_to_rereg) do
		for i,tab in pairs(v) do
			if not map.tile[ tab.tid ] then
				mapper._reg_tile(tab.tid, map, ta)
			end
		end
	end
end
