local write, writeIndent, writers, refCount;
local strbuff

persist =
{
	store = function (path, ...)
		strbuff = {}
		-- only if not empty path
		local file, e
		if '' ~= path then
			file, e = io.open(path, "w");
			if not file then
				return error(e);
			end
		end
		local n = select("#", ...);
		-- Count references
		local objRefCount = {}; -- Stores reference that will be exported
		for i = 1, n do
			refCount(objRefCount, (select(i,...)));
		end;
		-- Export Objects with more than one ref and assign name
		-- First, create empty tables for each
		local objRefNames = {};
		local objRefIdx = 0;
		table.insert(strbuff, "-- Persistent Data\n");
		table.insert(strbuff, "local multiRefObjects = {\n");
		for obj, count in pairs(objRefCount) do
			if count > 1 then
				objRefIdx = objRefIdx + 1;
				objRefNames[obj] = objRefIdx;
				table.insert(strbuff, "{};"); -- table objRefIdx
			end;
		end;
		table.insert(strbuff, "\n} -- multiRefObjects\n");
		-- Then fill them (this requires all empty multiRefObjects to exist)
		for obj, idx in pairs(objRefNames) do
			for k, v in pairs(obj) do
				table.insert(strbuff, "multiRefObjects["..idx.."][");
				write(file, k, 0, objRefNames);
				table.insert(strbuff, "] = ");
				write(file, v, 0, objRefNames);
				table.insert(strbuff, ";\n");
			end;
		end;
		-- Create the remaining objects
		for i = 1, n do
			table.insert(strbuff, "local ".."obj"..i.." = ");
			write(file, (select(i,...)), 0, objRefNames);
			table.insert(strbuff, "\n");
		end
		-- Return them
		if n > 0 then
			table.insert(strbuff, "return obj1");
			for i = 2, n do
				table.insert(strbuff, " ,obj"..i);
			end;
			table.insert(strbuff, "\n");
		else
			table.insert(strbuff, "return\n");
		end;
		local str =table.concat(strbuff)
		strbuff = {}
		if '' ~= path then
			file:write(str)
			file:close();
		else
			return str
		end
	end;

	load = function (path)
		local f, e = loadfile(path);
		if f then
			return f();
		else
			return nil, e;
		end;
	end;

	loadstr = function(str)
		local f = loadstring(str)
		if f then
			return f()
		else
			return nil
		end
	end;
}

-- Private methods

-- write thing (dispatcher)
write = function (file, item, level, objRefNames)
	writers[type(item)](file, item, level, objRefNames);
end;

-- write indent
writeIndent = function (file, level)
	for i = 1, level do
		table.insert(strbuff, "\t");
	end;
end;

-- recursively count references
refCount = function (objRefCount, item)
	-- only count reference types (tables)
	if type(item) == "table" then
		-- Increase ref count
		if objRefCount[item] then
			objRefCount[item] = objRefCount[item] + 1;
		else
			objRefCount[item] = 1;
			-- If first encounter, traverse
			for k, v in pairs(item) do
				refCount(objRefCount, k);
				refCount(objRefCount, v);
			end;
		end;
	end;
end;

-- Format items for the purpose of restoring
writers = {
	["nil"] = function (file, item)
			table.insert(strbuff, "nil");
		end;
	["number"] = function (file, item)
			table.insert(strbuff, tostring(item));
		end;
	["string"] = function (file, item)
			table.insert(strbuff, string.format("%q", item));
		end;
	["boolean"] = function (file, item)
			if item then
				table.insert(strbuff, "true");
			else
				table.insert(strbuff, "false");
			end
		end;
	["table"] = function (file, item, level, objRefNames)
			local refIdx = objRefNames[item];
			if refIdx then
				-- Table with multiple references
				table.insert(strbuff, "multiRefObjects["..refIdx.."]");
			else
				-- Single use table
				table.insert(strbuff, "{\n");
				for k, v in pairs(item) do
					writeIndent(file, level+1);
					table.insert(strbuff, "[");
					write(file, k, level+1, objRefNames);
					table.insert(strbuff, "] = ");
					write(file, v, level+1, objRefNames);
					table.insert(strbuff, ";\n");
				end
				writeIndent(file, level);
				table.insert(strbuff, "}");
			end;
		end;
	["function"] = function (file, item)
			-- Does only work for "normal" functions, not those
			-- with upvalues or c functions
			local dInfo = debug.getinfo(item, "uS");
			if dInfo.nups > 0 then
				table.insert(strbuff, "nil --[[functions with upvalue not supported]]");
			elseif dInfo.what ~= "Lua" then
				table.insert(strbuff, "nil --[[non-lua function not supported]]");
			else
				local r, s = pcall(string.dump,item);
				if r then
					table.insert(strbuff, string.format("loadstring(%q)", s));
				else
					table.insert(strbuff, "nil --[[function could not be dumped]]");
				end
			end
		end;
	["thread"] = function (file, item)
			table.insert(strbuff, "nil --[[thread]]\n");
		end;
	["userdata"] = function (file, item)
			table.insert(strbuff, "nil --[[userdata]]\n");
		end;
}

