-- musts:
--  require('watcher.lua') - doh
--	watcher.load(args)	--- to init watcher (in love.load())
--	watcher.draw()		--- to show stuff (in love.draw())
--	watcher.toggle(key, unicode)
--			=== to toggle debug screen with F10 (in love.keypressed())
--
-- optional:
--	[
--		watcher.update(dt)	--- to update coords (in love.update())
--		watcher.drag(x, y, button)		--- (in love.mousepressed())
--		watcher.undrag(x, y, button)	--- (in love.mousereleased())
--			=== allows debug window to be dragged with mouse
--			=== must use all 3 functions to work
--	]
--
--
-- [ only show information, if watcher is toggled with F10 ]
-- use in code:
--	watcher.watch(place, data_list ...)
--			=== call in update-like functions -	values refreshed a lot
--			=== doesn't retain information over frames
--			=== place - index to show in debug screen (1..n)
--			=== data_list - list of arguments to be concatenated with spaces
--	watcher.store(place, data_list ...)
--			=== call in load-like or keypress-like functions
--			=== retains information overframes
--			=== place - index to show in debug screen (1..n)
--			=== data_list - list of arguments to be concatenated with spaces
--	watcher.rect(x, y, width, height)
--			=== draws unicolor rectangle in place (use in *.draw()-like	functions
watcher = {}

function watcher.load(args)
	watcher.visible = 0
	watcher.width = 600
	watcher.height = 400
	watcher.wind_x = 50
	watcher.wind_y = 50
	watcher.title_y = -20
	watcher.data_x = 5
	watcher.data_y = 5
	watcher.fps_x = 2
	watcher.fps_y = 2
	watcher.space_v = 16
	watcher.space_h = 300
	watcher.dragd = nil
	watcher.data = {}
	watcher.storage = {}
	-- colors
	watcher.recColor = {200, 10, 50, 200}
	watcher.textColor = {10, 200, 50}
	watcher.textColor2 = {150, 100, 50}
	watcher.backColor1 = {200, 200, 200, 190}
	watcher.backColor2 = {10, 10, 50, 190}
	watcher.titleColor = {10, 150, 50, 150}
	watcher.fpsColor = {230, 10, 10}
end

function watcher.draw()
	if 0 ~= watcher.visible then
		local col = { love.graphics.getColor() }
		-- draws background
		love.graphics.setColor(watcher.backColor1)
		love.graphics.rectangle('fill',
					watcher.wind_x-2,  watcher.wind_y + watcher.title_y - 2,
					watcher.width+4,   watcher.height - watcher.title_y + 4)
		-- title
		love.graphics.setColor(watcher.titleColor)
		love.graphics.rectangle('fill', watcher.wind_x,  watcher.wind_y,
										watcher.width,   watcher.title_y)
		-- main
		love.graphics.setColor(watcher.backColor2)
		love.graphics.rectangle('fill', watcher.wind_x,	 watcher.wind_y,
										watcher.width,   watcher.height)
		-- draws watched values
		love.graphics.setColor(watcher.textColor)
		for _, d in ipairs(watcher.data) do
			love.graphics.print(d.info,
						watcher.wind_x + watcher.data_x + watcher.space_h * d.place[2],
						watcher.wind_y + watcher.data_y + watcher.space_v * d.place[1])
		end
		love.graphics.setColor(watcher.textColor2)
		for _, d in ipairs(watcher.storage) do
			love.graphics.print(d.info,
						watcher.wind_x + watcher.data_x + watcher.space_h * d.place[2],
						watcher.wind_y + watcher.data_y + watcher.space_v * d.place[1])
		end
		-- show fps ans some final touches
		love.graphics.setColor(watcher.fpsColor)
		love.graphics.print("FPS: " .. love.timer.getFPS(),
						watcher.wind_x + watcher.space_h,
						watcher.wind_y + watcher.title_y + 2)
		love.graphics.print('mem: ' .. collectgarbage('count') .. ' Kbytes',
						watcher.wind_x + 2,
						watcher.wind_y + watcher.title_y + 2)
		watcher.data = {}
		love.graphics.setColor(col)
	end
end

--function watcher.watch(place, every, possible, bit, of, information, available)
function watcher.watch(place, ... )
	if 0 ~= watcher.visible then
		local pl = type(place) == 'table' and place or {place, 0}
		local info = pl[1] .. ':'
		for i, v in ipairs(arg) do
			info = info .. ' ' .. (type(v) == 'table' and stprint(v) or v)
		end
		table.insert(watcher.data, {place = pl, info = info})
	end
end

function watcher.store(place, ... )
	local pl = type(place) == 'table' and place or {place, 0}
	local info = pl[1] .. ':'
	for i, v in ipairs(arg) do
		info = info .. ' ' .. (type(v) == 'table' and stprint(v) or v)
	end	
	for i, v in ipairs(watcher.storage) do
		if v.place[1] == pl[1] and v.place[2] == pl[2] then
			watcher.storage[i].info = info
			return
		end
	end
	table.insert(watcher.storage, {place = pl, info = info})
end

function watcher.rect(x, y, width, height)
	if 0 ~= watcher.visible then
		local col = { love.graphics.getColor() }
		love.graphics.setColor(watcher.recColor)
		love.graphics.rectangle('fill', x, y, width, height)
		love.graphics.setColor(col)
	end
end

function watcher.toggle(key, unicode)
	if key == 'f10' then
		watcher.visible = 1 - watcher.visible
	end
end

function watcher.update(dt)
	if type(watcher.dragd) == 'table' then
		watcher.wind_x = love.mouse.getX() - watcher.dragd.x
		watcher.wind_y = love.mouse.getY() - watcher.dragd.y
	end
end

function watcher.drag(x, y, button)
	if button == 'l' and
		x > watcher.wind_x and x < watcher.wind_x + watcher.width and
		y > watcher.wind_y + watcher.title_y and y < watcher.wind_y
	then
		watcher.dragd = {x = x - watcher.wind_x, y = y - watcher.wind_y}
	end
end

function watcher.undrag(x, y, button)
	if button == 'l' then
		watcher.dragd = nil
	end
end
