-- MUSTS:
--	require('skiplist.lua')
--	require('zbuff.lua')
--	buffer = zbuff(buff_size = 12)		--- creates zbuffer of size buff_size
--
--
-- use in code:
--  buffer:add(funct, z, function_arguments)
--			=== registers function for calling later in z order with given args
--			=== if z is the same for few such calls, earlier added functions
--			=== gets executed first
--  buffer:delete(z_index)		--- deletes all z_index'th layger
--	buffer[z_index]				--- returns an array of registered calls
--			=== if you only want to change function aguments
--			=== direct modification suffices:
--			=== local tmp = buffer[z_index]; tmp[i].args[argument_no] = new val
--			===
--			=== however if you want to change z order you have to:
--			=== 1. get values (if you need them)		[local t = buffer[z]]
--			=== 2. delete z'th layer					[bufffer:delete(z)]
--			=== 3. reinsert values			[for _,v in ipairs(t) do		]
--											[buffer:add(v.func,new_z,unpack(v))]
--											[end							]
--	buffer:clear()		--- clears all buffer
--	buffer:exec()		--- executes given functions with given args in z order
--	for k,v in buffer:ipairs()	--- iterates in execution order
--			=== v is in format: {v.func, v.z, v.args}
--			=== v.args being an array of given arguments

local function counter(self)
    self.__count = self.__count + 1
    return self.__count
end


local lt = function(a,b)
    if a.z < b.z then return true
    elseif a.z==b.z and a.id < b.id then return true
    else return false
    end
end

local mt={
    __lt=lt,
    __le=lt
}

local function add (self, func,z, ... )
--    local s={x = x, y = y, z = z, image=image, id = counter()}
	local s = {func = func, z = z, id = counter(self), args = {...} }

-- the id field is necessary because, otherwise, items on the same plane would cause troubles.
    self.slist:insert( setmetatable(s,mt) )
end


local function execute(self)
	for _,item in self.slist:ipairs() do
		item.func(unpack(item.args))
	end
end

local ipairs = function(self)
	return self.slist:ipairs()
end

local clear = function(self)
	self.slist = newskiplist(self.__maxsize)
	self.__count = 0
end

local MT = {
	__index = function(self, z)
		local t = {}
		local found = false
		for k,v in self.slist:ipairs() do
			if z < v.z then break end
			if z == v.z then
				found = true
				table.insert(t, v)
			end
		end
		return found and t or nil
	end,
	type = 'zbuff',
}

local replace = function(self, oldval, newval)
	self.slist:delete(oldval)
	self.slist:insert(setmetatable(newval, mt))
end

local delete = function(self, zindex)
	for k,v in self.slist:ipairs() do
		if zindex < v.z then break end
		if zindex == v.z then
			self.slist:delete(v)
		end
	end
end

function zbuff(size)
	local maxsize = size and size or 12
	local slist = newskiplist(maxsize)
	local count = 0

	return setmetatable({
		slist = slist,
		__count = count,
		__maxsize = maxsize,
		add = add,
		exec = execute,
		ipairs = ipairs,
		clear = clear,
		replace = replace,
		delete = delete,
	}, MT
)
end
