local hud = {
	active = -1,
	hasDynamic = {},
	hasStatic = {},
	bindList = {
		--dynamic objects elements
		earth = {x=1, y=2},
		water = {x=2, y=2},
		fire = {x=3, y=2},
		wind = {x=4, y=2},
		ice = {x=5, y=2},
		rectangle = {x=1, y=3},
		--static objects elements
		s_ice = {x=3, y=3},
		s_wind = {x=4, y=3},
		--heart parts (sorry for to much dynamics)
		h_tLeft = {x=1, y=4},
		h_tRight = {x=2, y=4},
		h_bLeft = {x=1, y=5},
		h_bRight = {x=2, y=5},
	},
	quads = {},
	--static elements padding
	staticPadX = love.graphics.getWidth()/1.15,
	staticPadY = love.graphics.getHeight()*0.04,
	--heart padding
	heartPadX = love.graphics.getWidth()*0.03,
	heartPadY = love.graphics.getHeight()*0.04,
	-- heat font
	heartFont = love.graphics.newFont( 25 ),
	exFont = nil, --font beforce change
	--dynamic elements  padding 
	dynamicPadX = love.graphics.getWidth()/1.1,
	dynamicPadY = love.graphics.getHeight()*0.04,
}
 --quad width
local qw = 32

--global finding procedure
local hasList = function (self, name, list)
	local hasValue = false
	for k,v in ipairs(list) do
		if v == name then
			hasValue = true
		end
	end
	return hasValue
end

local getActive = function(self)
	local hud = self.hud
	if hud.active ~= -1 then
		return hud.hasDynamic[hud.active]
	end
	return nil
end

--static list search procedure
local hasStatic = function (self, name)
	return hasList(self, name, self.hud.hasStatic)
end

--dynamic list search procedure
local hasDynamic = function (self, name)
	return hasList(self, name, self.hud.hasDynamic)
end

--pvz.: hud:addStatic("s_wind")
local addStatic = function (self, name)
	local hud = self.hud
	local hasValue = hasStatic(self, name)
	if not hasValue then
		local count = #hud.hasStatic
		hud.hasStatic[#hud.hasStatic+1] = name
	end
end

--pvz.: hud:addDynamic("ice")
local addDynamic = function (self, name)
	local hud = self.hud
	local hasValue = hasDynamic(self, name)
	if not hasValue then
		local count = #hud.hasDynamic
		if count == 0 then
			hud.active = 1
		end
		hud.hasDynamic[#hud.hasDynamic+1] = name
	end
end

local function acornUp(self, hero)
	if #(self.hud).hasDynamic then
		self.hud.active = (self.hud.active ) % #(self.hud).hasDynamic +1
	end
end

local function acornDown(self, hero)
	if #(self.hud).hasDynamic then
		self.hud.active = self.hud.active - 1
		if self.hud.active == 0 then
			self.hud.active = #(self.hud).hasDynamic
		end
	end
end

--draws heart from parts, to not load quads again
local drawHeart = function (self, dx, dy, hero)
	local hud = self.hud
	local qw = self.qw
	love.graphics.drawq(hud.img, 
		hud.quads[hud.bindList["h_tLeft"].y-1][hud.bindList["h_tLeft"].x-1],
		dx, dy)
	love.graphics.drawq(hud.img, 
		hud.quads[hud.bindList["h_tRight"].y-1][hud.bindList["h_tRight"].x-1],
		dx+32, dy)
	love.graphics.drawq(hud.img, 
		hud.quads[hud.bindList["h_bLeft"].y-1][hud.bindList["h_bLeft"].x-1],
		dx, dy+32)
	love.graphics.drawq(hud.img, 
		hud.quads[hud.bindList["h_bRight"].y-1][hud.bindList["h_bRight"].x-1],
		dx+32, dy+32)
	--draw number of lives
	love.graphics.setColor(0, 0, 0, 255 )
	hud.exFont = love.graphics.getFont()
	love.graphics.setFont( hud.heartFont )
	love.graphics.print(hero.health, dx+24, dy+19, 0, 1, 1)
	love.graphics.setFont( hud.exFont )
end


--draws all loaded quads
local drawAllPowerUps = function (self, img, quads, dx, dy)
	local hud = self.hud
	local qw = self.qw
	if quads ~= nil then
		--print(hud.quads)
		for x = 0, #quads do 
			for y = 0, #quads[x] do 
			--	print(":) x=".. x.. ", y=".. y) --debug
				love.graphics.drawq(img, quads[x][y], dx+y*qw, dy+x*qw)
			end
		end
	end
end

local drawList = function (self, img, quadList, showList, bindList, active, dx, dy)
	count = #showList
	for n = 1, count do
		hudX = bindList[showList[n]].x -1
		hudY = bindList[showList[n]].y -1
		x = dx - (count - n) * (qw + 6)
 		y = dy 
		love.graphics.drawq(img, quadList[hudY][hudX], x, y)
		--draws rectangle over active object
		--if n == hud.active then
			--love.graphics.rectangle("line", x, y, qw, qw) 
		--	love.graphics.drawq(hud.img, 
		--		hud.quads[hud.bindList["rectangle"].y-1][hud.bindList["rectangle"].x-1],
		--		 x, y)
		--end
	end
end

local drawDynamicList = function (self, img, quadList, showList, bindList, active, dx, dy)
	count = #showList
		--print(active)
	if active > 0 and count >= active then
		hudX = bindList[showList[active]].x -1
		hudY = bindList[showList[active]].y -1
		love.graphics.drawq(img, quadList[hudY][hudX], dx, dy)
	end
	love.graphics.drawq(img, 
		quadList[bindList["rectangle"].y-1][bindList["rectangle"].x-1],
		dx, dy)
end

local draw = function (self, hero)
	local hud = self.hud
	local qw = self.qw
	--draws all quads
	--drawAllPowerUps(self, hud.img, hud.quads, 0, 0)
	--draws static list of elements
	drawList(self, hud.img, hud.quads, hud.hasStatic, hud.bindList, -1, 
		hud.staticPadX,
		hud.staticPadY)
	--draws dynamic list of elements (available to be chosen)
	drawDynamicList(self, hud.img, hud.quads, hud.hasDynamic, hud.bindList, hud.active, 
		hud.dynamicPadX,
		hud.dynamicPadY)
	--draws heart
	drawHeart(self, hud.heartPadX, hud.heartPadY, hero)
end

local loadTransBMP = function(self, imgFile, tR, tG, tB)
	local imgSrc = love.image.newImageData(imgFile)
	imgSrc:mapPixel(
		function(x, y, r, g, b, a)
			if (r == tR and g == tG and b == tB) then
				a = 0
			end
			return r, g, b, a
		end
	)
	imgSrc = love.graphics.newImage(imgSrc)
	return imgSrc
end

local loadQuads = function(self, obj, imgFile, qw)
	--obj.img = love.graphics.newImage(imgFile)
	obj.img = loadTransBMP(self, imgFile, 0,0,0) --loads transparent img
	obj.quads = {}
	obj.qw = qw
	qcY = obj.img:getHeight() / qw - 1
	qcX = obj.img:getWidth() / qw - 1
	for y = 0, qcY  do
		obj.quads[y] = {}
		for x = 0, qcX do
			obj.quads[y][x] = love.graphics.newQuad(x * qw, y * qw, qw, qw, obj.img:getWidth(), obj.img:getHeight())
		end
	end
end

function newHud()
	loadQuads(nil, hud, "assets/consumables.bmp", 32)

	return setmetatable({
			draw = draw,
			hud = hud,
			quads = hud.quads,
			loadQuads = loadQuads,
			addDynamic = addDynamic,
			addStatic = addStatic,
			hasStatic = hasStatic,		--search function
			hasDynamic = hasDynamic,	--search function
			getActive = getActive,		--returns active name
			acornUp = acornUp,
			acornDown = acornDown,
			qw = qw,
		}, nil
	)
end

