local function updatewii(self, h, dt, hash)
	if self.start + self.lifetime < love.timer.getMicroTime() then
		hash:remove(h.m)
		h.m = nil
		return
	end
	local old_pos = self.pos:clone()
	self.dpos.x = self.dpos.x + self.speed  * dt
	self.pos = self.spos + self.dpos + self.offset
	self.p:setPosition(self.pos.x, self.pos.y)
	self.p:update(dt)
	hash:update(self, self.pos, self.old_pos)
end

local function drawwii(self, h)
	--[[
	love.graphics.setColor(0,0,0xff)
	love.graphics.rectangle('fill',
		self.pos.x - h.pos.x,
		self.pos.y - h.pos.y,
		self.shape.x, self.shape.y)
	]]
	love.graphics.setColor(0xff,0xff,0xff)
	love.graphics.setColorMode('modulate')
	love.graphics.setBlendMode('additive')
	love.graphics.draw(
		self.p,
		self.shape.x/2 - h.pos.x,
		self.shape.y- h.pos.y
	)
	love.graphics.setBlendMode('alpha')
end

function initwii(self, type, time)
	local m = {}
	m.img = love.graphics.newImage('assets/square.png')
	m.offset = vector(self.dir*70, -30)
	m.pos = vector(mapper.hscreen_w + (-self.shape.x/2 + m.offset.x) * self.dir, mapper.hscreen_h + m.offset.y)
	m.shape = vector(30, 90)
	m.dpos = vector(0,0)
	m.spos = hero.pos:clone()
	m.lifetime = time
	m.speed = 30 * self.dir
	m.start = love.timer.getMicroTime()
	local p = love.graphics.newParticleSystem(m.img, 300)
	p:setEmissionRate(060)
	p:setSpeed(300, 250)
	p:setGravity(200, 300)
	p:setSize(1, 2)
	p:setColor(140, 203, 176, 155, 104, 240, 3, 0)
	p:setPosition(400, 300)
	p:setLifetime(time)
	p:setParticleLife(1.0)
	p:setDirection(-3.14/2)
	p:setSpread(0)
	p:setSpin(300, 800)
	p:setRadialAcceleration(-200)
	p:setTangentialAcceleration(self.dir*200)
	p:stop()
	p:start()
	m.p = p
	m.updatem = updatewii
	m.drawm = drawwii
	hash:insert(m)
	m.type = 'gmagic'

	return setmetatable(m, {})
end

-- Yes, this is very not nice to do stuff...
-- takes hero ref, table of magic coords and dt
local function updatem(self, h, dt, hash)
	if self.start + self.lifetime  < love.timer.getMicroTime() then
		hash:remove(h.m)
		h.m = nil
		return
	end
	local old_pos = self.pos:clone()
	self.pos.x = h.pos.x + (h.shape.x/2 + self.offset.x) * h.dir
	self.pos.y = h.pos.y + h.shape.y/2 + self.offset.y
	hash:update(self, self.pos, old_pos)
--	m.p:setPosition(mapper.hscreen_w + (h.shape.x/2 + m.offset.x) * h.dir,mapper.hscreen_h + h.shape.y/2)
	self.p:setPosition(self.pos.x, self.pos.y)
	self.p:setDirection(1.58 - h.dir*1.18)
	self.p:update(dt)
end

-- takes hero ref, table of magic coords
local function drawm(self, h)
	--[[
	love.graphics.setColor(0, 0, 0xff)
	love.graphics.rectangle('fill',
		self.pos.x - h.pos.x,
		self.pos.y - h.pos.y,
		self.shape.x, self.shape.y)
	]]
	love.graphics.setColor(0xff,0xff,0xff)

		love.graphics.setColorMode("modulate")
		if self.type == 'fmagic' or self.type == 'imagic' then
			love.graphics.setBlendMode("additive")
		elseif self.type == 'wmagic' or self.type == 'emagic' then
			love.graphics.setBlendMode('alpha')
		end

		love.graphics.draw(
			self.p,
			self.shape.x/2 - self.shape.x/2 * h.dir - h.pos.x,
			- h.pos.y - self.offset.y
		)
		love.graphics.setBlendMode("alpha")
end

-- takes heor ref, and particle lifetime
local function initm(self, type, time)
	local m = {}
	if type == 'ice' then
		m.img = love.graphics.newImage('assets/stars.png')
	else
		m.img = love.graphics.newImage('assets/part1.png')
	end
	m.offset = vector(50, -40)
	m.pos = vector(mapper.hscreen_w + (self.shape.x/2 + m.offset.x) * self.dir, mapper.hscreen_h + self.shape.y/2 + m.offset.y)
	m.shape = vector(30, 80)
	m.lifetime = time
	m.start = love.timer.getMicroTime()
	local p = love.graphics.newParticleSystem(m.img, 150)
	if type == earth then
		p:setEmissionRate(100)
	else
		p:setEmissionRate(200)
	end
	p:setSpeed(100, 150)
	p:setSize(1, 1)
	if type == 'fire' then
		p:setColor(125, 25, 25, 255, 255, 128, 28, 0)
		m.type = 'fmagic'
	elseif type == 'water' then
		p:setColor(64, 128, 210, 255, 80, 100, 180, 75)
		m.type = 'wmagic'
	elseif type == 'earth' then
		p:setColor(179, 200, 12, 255, 200, 120, 12, 128)
		m.type = 'emagic'
	elseif type == 'ice' then
		p:setColor(64, 128, 210, 255, 80, 100, 180, 75)
		m.type = 'imagic'
	end
	p:setPosition(m.pos.x, m.pos.y + m.shape.y/2)
	p:setLifetime(time)
	p:setParticleLife(0.5)
	p:setDirection(1.58 - self.dir*1.18)
	p:setSpread(1.5)
	p:setRadialAcceleration(-100)
--	p:setTangentialAcceleration(500)
	p:stop()
	p:start()
	m.p = p
	m.updatem = updatem
	m.drawm = drawm
	hash:insert(m)

	return setmetatable(m, {})
end

return {
	initm = initm,
	initwii = initwii,
}
