-- requires
require('bitsnpieces/init')
require('AI')
require('hero')
require('conf')		-- config
require('animation')-- for animation of sprites
require('controlMenu')
require('hud')
require('audio')	-- sounds, music tracks
require('TEsound')	-- music library
require('ending')		-- ending
require('helpers')

-- paths
img = 'img/'
aud = 'audio/'

-- relatively small number
eps = 0.001

--gc('setpause', 5000)
gc('stop')
function love.load(args)
--	fi = io.open('log', 'w')
	GAME_STATE = "game" --game, control_menu, end
	-- in-game console
	watcher.load()
	-- fonts and directories
	sign_font = love.graphics.newFont(mapper.asset_path .. 'intuitive.ttf', 20)
	love.filesystem.setIdentity('semiou')
	love.filesystem.setIdentity('.')

	hero = newHero(400, 300)
	hero.controlOptions, hero.control, hero.controlPressed, hero.controlRelease = love.filesystem.load("bitsnpieces/TLbind.lua")() --oo approach
	controlMenu = newControlMenu(hero.controlOptions, hero.control)
	-- load lvl 1
	loadLvl('1', 2, 5, 0, 0)		-- load level 1, hero tile coord 5 5, bgxyoff 0 0	
--	loadLvl('3', 10, 19, 0, 0)		-- for debug
	
	love.joystick.open(0)
	
	-- sounds load
	audio:load()
	audio:play(1, true, 0.4)	-- start to play first bgm, loop, vol
	
	--Hud
	hud = newHud()
	ending = newEnding()
	-- local time vars
	_min_frame_time = 0.015
	_frame_sum_dt = 0
end

function love.update(dt)
	watcher.watch(6, hero.state)
	if GAME_STATE == "game" then
		_frame_sum_dt = _frame_sum_dt + dt
		-- I am not limiting fps, I am limiting times, updates gets called
		if _frame_sum_dt > _min_frame_time then -- limiting to minimum dt (on superfast PCs)
			-- in case we get lag, so we don't accidentally get out of map :)
			_frame_sum_dt = math.min(_frame_sum_dt, 0.05)

			hero.controlOptions.update(_frame_sum_dt) -- updates controls, oo approach

			-- hero movement
			if hero.control.left then
			local factor = 'jump' == hero.state and -0.3 or -1
				applyForce(hero, 700 * factor, 0)
			if hero.state == "stand" and hero.state ~= "jump" then
				hero.state = "walk"
			end
			hero.dir = -1
			end
			if hero.control.right then
			local factor = 'jump' == hero.state and 0.3 or 1
				applyForce(hero, 700 * factor, 0)
			if hero.state == "stand" and hero.state ~= "jump" then
				hero.state = "walk"
			end
			hero.dir = 1
			end

			AI:update(_frame_sum_dt, hash)
			hero:update(_frame_sum_dt, hash)
	
	
			watcher.update(_frame_sum_dt)
			
			
			-- hero stoped moving / jumping and attacks
			if hero.controlRelease.left or hero.controlRelease.right then
				local factor = 'jump' == hero.state and -0.6 or -0.99
				applyImpulse(hero, hero.vel.x * factor, 0)
				if 'jump' ~= hero.state then
					hero.state = "stand"
				end
			end
			if hero.controlPressed.jump and hero.jump_count < hero.max_jump then
				hero.vel.y = -hero:get_jump_power()
				hero.jump_count = hero.jump_count + 1
				hero.state = "jump"
				audio:play(5, false, 1)		-- jump sound
			end
			if hero.controlPressed.upsNext then
				hud:acornUp(hero)
			end
			if hero.controlPressed.upsBack then
				hud:acornDown(hero)
			end
			if hero.control.attack then
				local act = hud:getActive()
				if act then
					hero:domagic(act , mapper.acorns[act..' acorn'])
				end
			end

			-- sum new dt
			_frame_sum_dt = 0
		end -- limiting to minimum dt (on superfast PCs)
	
	end

	-- have to skip few frames after loading lvl, because laoding takes alot of
	-- time and causes some unwanted effects in physics
	if GAME_STATE == 'loading' then
		if skip_frame == 0 then
			GAME_STATE = 'game'
		end
		skip_frame = skip_frame - 1
	end
	if GAME_STATE == "control_menu" then
		controlMenu:update(dt)	--control menu
	end
	if GAME_STATE == "end" then
		ending:update(dt)
	end

	
	-- audio cleanup
	TEsound.cleanup()
	
	-- long awaited esc key
	if love.keyboard.isDown("escape") then
		love.event.push("q")
	end
end

function love.keypressed(key, uni)
	watcher.toggle(key, uni)
end

function love.mousepressed(x, y, but)
	watcher.drag(x, y, but)
end

function love.mousereleased(x, y, but)
	watcher.undrag(x, y, but)
end

function love.draw()
	if GAME_STATE == "game" then
		love.graphics.setColor(0xff,0xff,0xff)
		-- one translate for much of stuff
		love.graphics.push()
		local camx = clamp(hero.pos.x, mapper.hscreen_w, mapper.pmap_width - mapper.hscreen_w)
		local camy = clamp(hero.pos.y, mapper.hscreen_h, mapper.pmap_height - mapper.hscreen_h)
		love.graphics.translate(-camx, -camy)
		if bgImg_1 then
		love.graphics.draw(bgImg_1, 			-- bg source
			0, 0,								-- position
			0, 1, 1,		 					-- rotation radians, scaling x, scaling y
			- mapper.hscreen_w - bgxoff, - mapper.hscreen_h - bgyoff)	-- offset
		end
		if bgImg_2 then
		love.graphics.draw(bgImg_2,
			bgImg_1:getWidth(), 0,
			0, 1, 1,
			- mapper.hscreen_w - bgxoff, - mapper.hscreen_h - bgyoff)
		end

		mapper.draw_layer(camx, camy, 'background')
		mapper.draw_layer(camx, camy, 'decorations')
		AI:draw(camx, camy)
--		draw_objects(camx, camy)
		mapper.draw_objects_directly(camx, camy)

		-- hero gets drawn by bit different laws, so we have pop matrix
		love.graphics.pop()

		hero:draw()

		-- ok, 2 translates, because we have other thing we want to draw in the
		-- same place as before
		love.graphics.push()
		love.graphics.translate(-camx, -camy)

		mapper.draw_layer(camx, camy, 'foreground')
		mapper.show_sign(hero.sign)

		mapper.drawCollisions(hero.pos.x, hero.pos.y)
		love.graphics.pop()

		watcher.draw()
		hud:draw(hero)
	end
	if GAME_STATE == "control_menu" then
		controlMenu:draw()	-- control menu
	end
	if GAME_STATE == "end" then
		ending:draw()
	end
end

-- to exit
function love.quit()
	audio:stop(1)			-- stop 1 bgm
	cleanup_lvldata()
end
